﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace TechNote33_FileCopy_CS
{
    class FileCopyCoreEx
    {
        public FileCopyCoreEx()
        {
            m_bFileCopyOverWrite = false;
        }

        private bool m_bFileCopyOverWrite;
        public bool FileOverWrite
        {
            set
            {
                m_bFileCopyOverWrite = value;
            }
            get
            {
                return m_bFileCopyOverWrite;
            }
        }

        // 소스파일의 원본 경로, 목적지 경로를 설정하고 사용자가 설정한 재시도 횟수만큼 복사를 시도하고 복사 실패시 해당 파일을 CopyFailList에 추가하고 해당 정보를 반환
        // 인자로는 SrcFile : 원본 경로, DestFile : 목적지 경로, filelist : 파일 리스트, iRetryCnt : 파일 복사 재시도 횟수, iSuccessCnt : 복사 성공 횟수
        public string[] FileCopyRetry(string strSrcFile, string strDestFile, string[] filelist, int iRetryCnt, out int iSuccessCnt)
        {
            // 파일 복사 실패 리스트 저장 배열
            List<string> lstCopyFailList = new List<string>();

            // 파일 복사 재시도 횟수, 성공 횟수 초기화
            int iRCnt = 0;
            iSuccessCnt = 0;

            DirectoryInfo di = new DirectoryInfo(strDestFile);
            if (di.Exists == false)
            {
                // 파일이 복사될 위치에 폴더가 없는 경우 생성
                di.Create();
            }

            // 리스트 카운트만큼 반복
            for (int i = 0; i < filelist.Length; i++)
            {
                // strSrcFile + filelist[i] 은 "Flash Disk\\Src\\" + "1.txt" 로 표시됨
                // strDestFile + filelist[i] 은 "SD Card\\Dest\\" + "1.txt" 로 표시됨
                if (FileCopy(strSrcFile + filelist[i], strDestFile + filelist[i]) == false)
                {
                    // 파일 복사 실패하는경우
                    // 재시도 카운트 1증가
                    ++iRCnt;
                    if (iRetryCnt <= iRCnt)
                    {
                        // 재시도 카운트가 사용자가 설정한 카운트라면
                        // 사용자가 설정한 카운트 만큼 반복해도 파일 복사 실패시 해당 파일을 lstCopyFailList에 추가
                        lstCopyFailList.Add(filelist[i]);
                    }
                    else
                    {
                        // 재시도 카운트가 사용자가 설정한 카운트보다 작은 경우
                        // 동일한 파일의 복사를 재시도 해야하므로 i값은 증가하면 안됨
                        i--;
                        // 상기 for문으로 이동하여 FileCopy 재시도 수행
                        continue;
                    }
                }
                else
                {
                    // 파일 복사가 성공하게되는 경우
                    // 성공 카운트 증가
                    iSuccessCnt++;
                    // 재시도 카운트는 0으로 초기화
                    iRCnt = 0;
                }
            }

            // 파일 복사 실패를 배열로 리턴
            return lstCopyFailList.ToArray();
        }

        // 파일 복사를 수행할 메서드. 복사성공시 TRUE반환, 복사실패시 FALSE반환
        private bool FileCopy(string source, string dest)
        {
            try
            {
                // 1. File.Copy메서드
                // source 경로의 파일을 dest경로로 복사하되 동일이름의 파일은 덮어쓰기가 실패합니다.
                // (인자)source : 복사할 원본파일, (인자)dest : 복사될 경로의 대상파일이름
                // File.Copy(source, dest);

                // 2. File.Copy메서드의 오버로드(덮어쓰기 기능 포함)
                // source 경로의 파일을 dest경로로 복사하되 m_bFileCopyOverWrite가 True인경우 동일이름의 파일을 덮어쓰기가 가능
                // (인자)m_bFileCopyOverWrite : True(덮어쓰기가능), False(덮어쓰기불가능)
                File.Copy(source, dest, m_bFileCopyOverWrite);
            }
            // FileNotFoundException 예외 : source경로의 FileName을 찾을 수 없는경우 예외발생
            // 디버깅 모드로 트레이싱하는 경우 source 경로의 파일에 문제가 있는 경우
            // FileNotFoundException 예외처리를 해주어 source 경로의 파일 문제를 쉽게 찾을수 있음.
            // 릴리즈모드로 배포하는 경우 false 리턴.
            catch (FileNotFoundException)
            {
#if DEBUG
                throw new System.IO.FileNotFoundException(source + " File not Found");
#else
                return false;
#endif
            }
            catch (IOException)
            {
                return false;
            }
            return true;
        }
    }
}
