﻿Imports System.IO
Imports System.Runtime.InteropServices

Public Class FileCopyCoreEx
    Public Sub New()
        m_bFileCopyOverWrite = False
    End Sub

    Private m_bFileCopyOverWrite As Boolean

    Public Property FileOverWrite() As Boolean
        Set(ByVal value As Boolean)
            m_bFileCopyOverWrite = value
        End Set
        Get
            Return m_bFileCopyOverWrite
        End Get
    End Property

    ' 소스파일의 원본 경로, 목적지 경로를 설정하고 사용자가 설정한 재시도 횟수만큼 복사를 시도하고 복사 실패시 해당 파일을 CopyFailList에 추가하고 해당 정보를 반환
    ' 인자로는 SrcFile : 원본 경로, DestFile : 목적지 경로, filelist : 파일 리스트, iRetryCnt : 파일 복사 재시도 횟수, iSuccessCnt : 복사 성공 횟수
    Public Function FileCopyRetry(ByVal strSrcFile As String, ByVal strDestFile As String, ByVal filelist As String(), ByVal iRetryCnt As Integer, ByRef iSuccessCnt As Integer) As String()
        ' 파일 복사 실패 리스트 저장 배열
        Dim lstCopyFailList As List(Of String) = New List(Of String)()

        ' 파일 복사 재시도 횟수, 성공 횟수 초기화
        Dim iRCnt As Integer = 0
        iSuccessCnt = 0

        Dim di As DirectoryInfo = New DirectoryInfo(strDestFile)
        If di.Exists = False Then
            ' 파일이 복사될 위치에 폴더가 없는 경우 생성
            di.Create()
        End If

        ' 리스트 카운트만큼 반복
        For i As Integer = 0 To filelist.Length - 1
            ' strSrcFile + filelist[i] 은 "Flash Disk\\Src\\" + "1.txt" 로 표시됨
            ' strDestFile + filelist[i] 은 "SD Card\\Dest\\" + "1.txt" 로 표시됨
            If FileCopy(strSrcFile & filelist(i), strDestFile & filelist(i)) = False Then
                ' 파일 복사 실패하는경우
                ' 재시도 카운트 1증가
                iRCnt += 1
                If iRetryCnt <= iRCnt Then
                    ' 재시도 카운트가 사용자가 설정한 카운트라면
                    ' 사용자가 설정한 카운트 만큼 반복해도 파일 복사 실패시 해당 파일을 lstCopyFailList에 추가
                    lstCopyFailList.Add(filelist(i))
                Else
                    ' 재시도 카운트가 사용자가 설정한 카운트보다 작은 경우
                    ' 동일한 파일의 복사를 재시도 해야하므로 i값은 증가하면 안됨
                    i -= 1
                    ' 상기 for문으로 이동하여 FileCopy 재시도 수행
                    Continue For
                End If
            Else
                ' 파일 복사가 성공하게되는 경우
                ' 성공 카운트 증가
                iSuccessCnt += 1
                ' 재시도 카운트는 0으로 초기화
                iRCnt = 0
            End If
        Next

        ' 파일 복사 실패를 배열로 리턴
        Return lstCopyFailList.ToArray()
    End Function

    ' 파일 복사를 수행할 메서드. 복사성공시 TRUE반환, 복사실패시 FALSE반환
    Private Function FileCopy(ByVal source As String, ByVal dest As String) As Boolean
        Try
            ' 1. File.Copy메서드
            ' source 경로의 파일을 dest경로로 복사하되 동일이름의 파일은 덮어쓰기가 실패합니다.
            ' (인자)source : 복사할 원본파일, (인자)dest : 복사될 경로의 대상파일이름
            ' File.Copy(source, dest);

            ' 2. File.Copy메서드의 오버로드(덮어쓰기 기능 포함)
            ' source 경로의 파일을 dest경로로 복사하되 m_bFileCopyOverWrite가 True인경우 동일이름의 파일을 덮어쓰기가 가능
            ' (인자)m_bFileCopyOverWrite : True(덮어쓰기가능), False(덮어쓰기불가능)
            File.Copy(source, dest, m_bFileCopyOverWrite)
        Catch __unusedFileNotFoundException1__ As FileNotFoundException
            ' FileNotFoundException 예외 : source경로의 FileName을 찾을 수 없는경우 예외발생
            ' 디버깅 모드로 트레이싱하는 경우 source 경로의 파일에 문제가 있는 경우
            ' FileNotFoundException 예외처리를 해주어 source 경로의 파일 문제를 쉽게 찾을수 있음.
            ' 릴리즈모드로 배포하는 경우 false 리턴.
#If DEBUG Then
            Throw New System.IO.FileNotFoundException(source + " File not Found")
#Else
            Return False
#End If
        Catch __unusedIOException2__ As IOException
            Return False
        End Try

        Return True
    End Function
End Class
